@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2021-2024, by Samuel Williams.
+# Copyright, 2021-2025, by Samuel Williams.
 # Copyright, 2023, by Math Ieu.
 
 require_relative "../interrupt"
@@ -199,7 +199,7 @@ module IO::Event
 							result = Fiber.blocking{buffer.read(io, 0, offset)}
 							
 							if result < 0
-								if again?(result)
+								if length > 0 and again?(result)
 									self.io_wait(fiber, io, IO::READABLE)
 								else
 									return result
@@ -229,7 +229,7 @@ module IO::Event
 							result = Fiber.blocking{buffer.write(io, 0, offset)}
 							
 							if result < 0
-								if again?(result)
+								if length > 0 and again?(result)
 									self.io_wait(fiber, io, IO::READABLE)
 								else
 									return result
@@ -305,96 +305,14 @@ module IO::Event
 					
 					return total
 				end
-			elsif Support.fiber_scheduler_v1?
-				# Ruby <= 3.1, limited IO::Buffer support.
-				def io_read(fiber, _io, buffer, length, offset = 0)
-					# We need to avoid any internal buffering, so we use a duplicated IO object:
-					io = IO.for_fd(_io.fileno, autoclose: false)
-					
-					total = 0
-					
-					maximum_size = buffer.size - offset
-					while maximum_size > 0
-						case result = blocking{io.read_nonblock(maximum_size, exception: false)}
-						when :wait_readable
-							if length > 0
-								self.io_wait(fiber, io, IO::READABLE)
-							else
-								return EWOULDBLOCK
-							end
-						when :wait_writable
-							if length > 0
-								self.io_wait(fiber, io, IO::WRITABLE)
-							else
-								return EWOULDBLOCK
-							end
-						when nil
-							break
-						else
-							buffer.set_string(result, offset)
-							
-							size = result.bytesize
-							total += size
-							offset += size
-							break if size >= length
-							length -= size
-						end
-						
-						maximum_size = buffer.size - offset
-					end
-					
-					return total
-				rescue IOError => error
-					return -Errno::EBADF::Errno
-				rescue SystemCallError => error
-					return -error.errno
-				end
-				
-				def io_write(fiber, _io, buffer, length, offset = 0)
-					# We need to avoid any internal buffering, so we use a duplicated IO object:
-					io = IO.for_fd(_io.fileno, autoclose: false)
-					
-					total = 0
-					
-					maximum_size = buffer.size - offset
-					while maximum_size > 0
-						chunk = buffer.get_string(offset, maximum_size)
-						case result = blocking{io.write_nonblock(chunk, exception: false)}
-						when :wait_readable
-							if length > 0
-								self.io_wait(fiber, io, IO::READABLE)
-							else
-								return EWOULDBLOCK
-							end
-						when :wait_writable
-							if length > 0
-								self.io_wait(fiber, io, IO::WRITABLE)
-							else
-								return EWOULDBLOCK
-							end
-						else
-							total += result
-							offset += result
-							break if result >= length
-							length -= result
-						end
-						
-						maximum_size = buffer.size - offset
-					end
-					
-					return total
-				rescue IOError => error
-					return -Errno::EBADF::Errno
-				rescue SystemCallError => error
-					return -error.errno
-				end
-				
-				def blocking(&block)
-					fiber = Fiber.new(blocking: true, &block)
-					return fiber.resume(fiber)
-				end
 			end
 			
+			# Wait for a process to change state.
+			#
+			# @parameter fiber [Fiber] The fiber to resume after waiting.
+			# @parameter pid [Integer] The process ID to wait for.
+			# @parameter flags [Integer] Flags to pass to Process::Status.wait.
+			# @returns [Process::Status] The status of the waited process.
 			def process_wait(fiber, pid, flags)
 				Thread.new do
 					Process::Status.wait(pid, flags)
@@ -414,6 +332,10 @@ module IO::Event
 				end
 			end
 			
+			# Wait for IO events or a timeout.
+			#
+			# @parameter duration [Numeric | Nil] The maximum time to wait, or nil for no timeout.
+			# @returns [Integer] The number of ready IO objects.
 			def select(duration = nil)
 				if pop_ready
 					# If we have popped items from the ready list, they may influence the duration calculation, so we don't delay the event loop:
